@@ -12,7 +12,7 @@ from xonsh.lazyasd import lazyobject
 from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
 from xonsh.environ import locate_binary, make_args_env
 from xonsh.foreign_shells import foreign_shell_data
-from xonsh.jobs import jobs, fg, bg, clean_jobs
+from xonsh.jobs import jobs, fg, bg, clean_jobs, disown
 from xonsh.platform import (
     ON_ANACONDA,
     ON_DARWIN,
@@ -48,8 +48,8 @@ if ON_POSIX:
 
 
 @lazyobject
-def SUB_EXEC_ALIAS_RE():
-    return re.compile(r"@\(|\$\(|!\(|\$\[|!\[|\&\&|\|\||\s+and\s+|\s+or\s+")
+def EXEC_ALIAS_RE():
+    return re.compile(r"@\(|\$\(|!\(|\$\[|!\[|\&\&|\|\||\s+and\s+|\s+or\s+|[>|<]")
 
 
 class Aliases(cabc.MutableMapping):
@@ -128,8 +128,8 @@ class Aliases(cabc.MutableMapping):
     def __setitem__(self, key, val):
         if isinstance(val, str):
             f = "<exec-alias:" + key + ">"
-            if SUB_EXEC_ALIAS_RE.search(val) is not None:
-                # We have a sub-command, e.g. $(cmd), to evaluate
+            if EXEC_ALIAS_RE.search(val) is not None:
+                # We have a sub-command (e.g. $(cmd)) or IO redirect (e.g. >>)
                 self._raw[key] = ExecAlias(val, filename=f)
             elif isexpression(val):
                 # expansion substitution
@@ -141,6 +141,23 @@ class Aliases(cabc.MutableMapping):
         else:
             self._raw[key] = val
 
+    def _common_or(self, other):
+        new_dict = self._raw.copy()
+        for key in dict(other):
+            new_dict[key] = other[key]
+        return Aliases(new_dict)
+
+    def __or__(self, other):
+        return self._common_or(other)
+
+    def __ror__(self, other):
+        return self._common_or(other)
+
+    def __ior__(self, other):
+        for key in dict(other):
+            self[key] = other[key]
+        return self
+
     def __delitem__(self, key):
         del self._raw[key]
 
@@ -770,6 +787,7 @@ def make_default_aliases():
         "jobs": jobs,
         "fg": fg,
         "bg": bg,
+        "disown": disown,
         "EOF": xonsh_exit,
         "exit": xonsh_exit,
         "quit": xonsh_exit,
