@@ -125,7 +125,7 @@ from ast import (
 from xonsh.platform import PYTHON_VERSION_INFO
 
 if PYTHON_VERSION_INFO > (3, 8):
-    from ast import NamedExpr
+    from ast import NamedExpr  # type:ignore
 
 STATEMENTS = (
     FunctionDef,
