@@ -1,15 +1,15 @@
 """Amalgamation of xonsh.completers package, made up of the following modules, in order:
 
 * bash_completion
-* completer
 * pip
 * tools
 * xompletions
-* _aliases
 * commands
+* completer
 * man
 * path
 * python
+* _aliases
 * base
 * bash
 * dirs
@@ -78,6 +78,7 @@ pathlib = _LazyModule.load('pathlib', 'pathlib')
 platform = _LazyModule.load('platform', 'platform')
 functools = _LazyModule.load('functools', 'functools')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
+tp = _LazyModule.load('typing', 'typing', 'tp')
 __version__ = "0.2.6"
 
 
@@ -162,7 +163,7 @@ def _bash_completion_paths_default():
     return bcd
 
 
-_BASH_COMPLETIONS_PATHS_DEFAULT = None
+_BASH_COMPLETIONS_PATHS_DEFAULT: tp.Tuple[str, ...] = ()
 
 
 def _get_bash_completions_source(paths=None):
@@ -178,7 +179,7 @@ def _get_bash_completions_source(paths=None):
 
 
 def _bash_get_sep():
-    """ Returns the appropriate filepath separator char depending on OS and
+    """Returns the appropriate filepath separator char depending on OS and
     xonsh options set
     """
     if platform.system() == "Windows":
@@ -187,7 +188,7 @@ def _bash_get_sep():
         return os.sep
 
 
-_BASH_PATTERN_NEED_QUOTES = None
+_BASH_PATTERN_NEED_QUOTES: tp.Optional[tp.Pattern] = None
 
 
 def _bash_pattern_need_quotes():
@@ -533,50 +534,6 @@ def _bc_main(args=None):
 if __name__ == "__main__":
     _bc_main()
 
-#
-# completer
-#
-builtins = _LazyModule.load('builtins', 'builtins')
-def complete_completer(prefix, line, start, end, ctx):
-    """
-    Completion for "completer"
-    """
-    args = line.split(" ")
-    if len(args) == 0 or args[0] != "completer":
-        return None
-
-    if end < len(line) and line[end] != " ":
-        # completing in a middle of a word
-        # (e.g. "completer some<TAB>thing")
-        return None
-
-    curix = args.index(prefix)
-
-    compnames = set(builtins.__xonsh__.completers.keys())
-    if curix == 1:
-        possible = {"list", "help", "add", "remove"}
-    elif curix == 2:
-        if args[1] == "help":
-            possible = {"list", "add", "remove"}
-        elif args[1] == "remove":
-            possible = compnames
-        else:
-            raise StopIteration
-    else:
-        if args[1] != "add":
-            raise StopIteration
-        if curix == 3:
-            possible = {i for i, j in builtins.__xonsh__.ctx.items() if callable(j)}
-        elif curix == 4:
-            possible = (
-                {"start", "end"}
-                | {">" + n for n in compnames}
-                | {"<" + n for n in compnames}
-            )
-        else:
-            raise StopIteration
-    return {i for i in possible if i.startswith(prefix)}
-
 #
 # pip
 #
@@ -638,7 +595,7 @@ def complete_pip(prefix, line, begidx, endidx, ctx):
 # tools
 #
 """Xonsh completer tools."""
-# amalgamated builtins
+builtins = _LazyModule.load('builtins', 'builtins')
 textwrap = _LazyModule.load('textwrap', 'textwrap')
 def _filter_normal(s, x):
     return s.startswith(x)
@@ -687,21 +644,19 @@ class RichCompletion(str):
         Extra text to display when the completion is selected.
     """
 
-    def __new__(cls, value, prefix_len=None, display=None, description=""):
+    def __new__(cls, value, prefix_len=None, display=None, description="", style=""):
         completion = super().__new__(cls, value)
 
         completion.prefix_len = prefix_len
         completion.display = display or value
         completion.description = description
+        completion.style = style
 
         return completion
 
     def __repr__(self):
         return "RichCompletion({}, prefix_len={}, display={}, description={})".format(
-            repr(str(self)),
-            self.prefix_len,
-            repr(self.display),
-            repr(self.description),
+            repr(str(self)), self.prefix_len, repr(self.display), repr(self.description)
         )
 
 
@@ -728,7 +683,11 @@ def get_ptk_completer():
 """Provides completions for xonsh internal utilities"""
 
 xx = _LazyModule.load('xonsh', 'xonsh.xontribs', 'xx')
+xmt = _LazyModule.load('xonsh', 'xonsh.xontribs_meta', 'xmt')
 xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
+from xonsh.xonfig import XONFIG_MAIN_ACTIONS
+
+
 def complete_xonfig(prefix, line, start, end, ctx):
     """Completion for ``xonfig``"""
     args = line.split(" ")
@@ -736,7 +695,7 @@ def complete_xonfig(prefix, line, start, end, ctx):
         return None
     curix = args.index(prefix)
     if curix == 1:
-        possible = {"info", "wizard", "styles", "colors", "-h"}
+        possible = set(XONFIG_MAIN_ACTIONS.keys()) | {"-h"}
     elif curix == 2 and args[1] == "colors":
         possible = set(xt.color_style_names())
     else:
@@ -745,10 +704,9 @@ def complete_xonfig(prefix, line, start, end, ctx):
 
 
 def _list_installed_xontribs():
-    meta = xx.xontrib_metadata()
+    meta = xmt.get_xontribs()
     installed = []
-    for md in meta["xontribs"]:
-        name = md["name"]
+    for name in meta:
         spec = xx.find_xontrib(name)
         if spec is not None:
             installed.append(spec.name.rsplit(".")[-1])
@@ -772,196 +730,6 @@ def complete_xontrib(prefix, line, start, end, ctx):
 
     return {i for i in possible if i.startswith(prefix)}
 
-#
-# _aliases
-#
-# amalgamated builtins
-collections = _LazyModule.load('collections', 'collections')
-# amalgamated xonsh.lazyasd
-# amalgamated xonsh.completers.tools
-VALID_ACTIONS = xl.LazyObject(
-    lambda: frozenset({"add", "remove", "list"}), globals(), "VALID_ACTIONS"
-)
-
-
-def _add_one_completer(name, func, loc="end"):
-    new = collections.OrderedDict()
-    if loc == "start":
-        new[name] = func
-        for (k, v) in builtins.__xonsh__.completers.items():
-            new[k] = v
-    elif loc == "end":
-        for (k, v) in builtins.__xonsh__.completers.items():
-            new[k] = v
-        new[name] = func
-    else:
-        direction, rel = loc[0], loc[1:]
-        found = False
-        for (k, v) in builtins.__xonsh__.completers.items():
-            if rel == k and direction == "<":
-                new[name] = func
-                found = True
-            new[k] = v
-            if rel == k and direction == ">":
-                new[name] = func
-                found = True
-        if not found:
-            new[name] = func
-    builtins.__xonsh__.completers.clear()
-    builtins.__xonsh__.completers.update(new)
-
-
-def _list_completers(args, stdin=None, stack=None):
-    o = "Registered Completer Functions: \n"
-    _comp = builtins.__xonsh__.completers
-    ml = max((len(i) for i in _comp), default=0)
-    _strs = []
-    for c in _comp:
-        if _comp[c].__doc__ is None:
-            doc = "No description provided"
-        else:
-            doc = " ".join(_comp[c].__doc__.split())
-        doc = justify(doc, 80, ml + 3)
-        _strs.append("{: >{}} : {}".format(c, ml, doc))
-    return o + "\n".join(_strs) + "\n"
-
-
-def _remove_completer(args, stdin=None, stack=None):
-    err = None
-    if len(args) != 1:
-        err = "completer remove takes exactly 1 argument."
-    else:
-        name = args[0]
-        if name not in builtins.__xonsh__.completers:
-            err = ("The name %s is not a registered " "completer function.") % name
-    if err is None:
-        del builtins.__xonsh__.completers[name]
-        return
-    else:
-        return None, err + "\n", 1
-
-
-def _register_completer(args, stdin=None, stack=None):
-    err = None
-    if len(args) not in {2, 3}:
-        err = (
-            "completer add takes either 2 or 3 arguments.\n"
-            "For help, run:  completer help add"
-        )
-    else:
-        name = args[0]
-        func_name = args[1]
-        if name in builtins.__xonsh__.completers:
-            err = ("The name %s is already a registered " "completer function.") % name
-        else:
-            if func_name in builtins.__xonsh__.ctx:
-                func = builtins.__xonsh__.ctx[func_name]
-                if not callable(func):
-                    err = "%s is not callable" % func_name
-            else:
-                for frame_info in stack:
-                    frame = frame_info[0]
-                    if func_name in frame.f_locals:
-                        func = frame.f_locals[func_name]
-                        break
-                    elif func_name in frame.f_globals:
-                        func = frame.f_globals[func_name]
-                        break
-                else:
-                    err = "No such function: %s" % func_name
-    if err is None:
-        position = "start" if len(args) == 2 else args[2]
-        _add_one_completer(name, func, position)
-    else:
-        return None, err + "\n", 1
-
-
-def completer_alias(args, stdin=None, stdout=None, stderr=None, spec=None, stack=None):
-    err = None
-    if len(args) == 0 or args[0] not in (VALID_ACTIONS | {"help"}):
-        err = (
-            "Please specify an action.  Valid actions are: "
-            '"add", "remove", "list", or "help".'
-        )
-    elif args[0] == "help":
-        if len(args) == 1 or args[1] not in VALID_ACTIONS:
-            return (
-                "Valid actions are: add, remove, list.  For help with a "
-                "specific action, run: completer help ACTION\n"
-            )
-        elif args[1] == "add":
-            return COMPLETER_ADD_HELP_STR
-        elif args[1] == "remove":
-            return COMPLETER_REMOVE_HELP_STR
-        elif args[1] == "list":
-            return COMPLETER_LIST_HELP_STR
-
-    if err is not None:
-        return None, err + "\n", 1
-
-    if args[0] == "add":
-        func = _register_completer
-    elif args[0] == "remove":
-        func = _remove_completer
-    elif args[0] == "list":
-        func = _list_completers
-    return func(args[1:], stdin=stdin, stack=stack)
-
-
-COMPLETER_LIST_HELP_STR = """completer list: ordered list the active completers
-
-Usage:
-    completer remove
-"""
-
-COMPLETER_REMOVE_HELP_STR = """completer remove: removes a completer from xonsh
-
-Usage:
-    completer remove NAME
-
-NAME is a unique name of a completer (run "completer list" to see the current
-     completers in order)
-"""
-
-COMPLETER_ADD_HELP_STR = """completer add: adds a new completer to xonsh
-
-Usage:
-    completer add NAME FUNC [POS]
-
-NAME is a unique name to use in the listing (run "completer list" to see the
-     current completers in order)
-
-FUNC is the name of a completer function to use.  This should be a function
-     of the following arguments, and should return a set of valid completions
-     for the given prefix.  If this completer should not be used in a given
-     context, it should return an empty set or None.
-
-     Arguments to FUNC:
-       * prefix: the string to be matched
-       * line: a string representing the whole current line, for context
-       * begidx: the index at which prefix starts in line
-       * endidx: the index at which prefix ends in line
-       * ctx: the current Python environment
-
-     If the completer expands the prefix in any way, it should return a tuple
-     of two elements: the first should be the set of completions, and the
-     second should be the length of the modified prefix (for an example, see
-     xonsh.completers.path.complete_path).
-
-POS (optional) is a position into the list of completers at which the new
-     completer should be added.  It can be one of the following values:
-       * "start" indicates that the completer should be added to the start of
-                 the list of completers (it should be run before all others)
-       * "end" indicates that the completer should be added to the end of the
-               list of completers (it should be run after all others)
-       * ">KEY", where KEY is a pre-existing name, indicates that this should
-                 be added after the completer named KEY
-       * "<KEY", where KEY is a pre-existing name, indicates that this should
-                 be added before the completer named KEY
-
-     If POS is not provided, the default value is "start"
-"""
-
 #
 # commands
 #
@@ -1027,6 +795,121 @@ def complete_skipper(cmd, line, start, end, ctx):
         cmd, " ".join(parts[skip_part_num:]), start - skip_len, end - skip_len, ctx
     )
 
+#
+# completer
+#
+# amalgamated builtins
+collections = _LazyModule.load('collections', 'collections')
+# amalgamated xonsh.completers.tools
+xla = _LazyModule.load('xonsh', 'xonsh.lazyasd', 'xla')
+@xla.lazyobject
+def xsh_session():
+    """return current xonshSession instance."""
+    return builtins.__xonsh__  # type: ignore
+
+
+def complete_completer(prefix, line, start, end, ctx):
+    """
+    Completion for "completer"
+    """
+    args = line.split(" ")
+    if len(args) == 0 or args[0] != "completer":
+        return None
+
+    if end < len(line) and line[end] != " ":
+        # completing in a middle of a word
+        # (e.g. "completer some<TAB>thing")
+        return None
+
+    curix = args.index(prefix)
+
+    compnames = set(xsh_session.completers.keys())
+    if curix == 1:
+        possible = {"list", "help", "add", "remove"}
+    elif curix == 2:
+        if args[1] == "help":
+            possible = {"list", "add", "remove"}
+        elif args[1] == "remove":
+            possible = compnames
+        else:
+            raise StopIteration
+    else:
+        if args[1] != "add":
+            raise StopIteration
+        if curix == 3:
+            possible = {i for i, j in xsh_session.ctx.items() if callable(j)}
+        elif curix == 4:
+            possible = (
+                {"start", "end"}
+                | {">" + n for n in compnames}
+                | {"<" + n for n in compnames}
+            )
+        else:
+            raise StopIteration
+    return {i for i in possible if i.startswith(prefix)}
+
+
+def add_one_completer(name, func, loc="end"):
+    new = collections.OrderedDict()
+    if loc == "start":
+        new[name] = func
+        for (k, v) in xsh_session.completers.items():
+            new[k] = v
+    elif loc == "end":
+        for (k, v) in xsh_session.completers.items():
+            new[k] = v
+        new[name] = func
+    else:
+        direction, rel = loc[0], loc[1:]
+        found = False
+        for (k, v) in xsh_session.completers.items():
+            if rel == k and direction == "<":
+                new[name] = func
+                found = True
+            new[k] = v
+            if rel == k and direction == ">":
+                new[name] = func
+                found = True
+        if not found:
+            new[name] = func
+    xsh_session.completers.clear()
+    xsh_session.completers.update(new)
+
+
+def list_completers():
+    """List the active completers"""
+    o = "Registered Completer Functions: \n"
+    _comp = xsh_session.completers
+    ml = max((len(i) for i in _comp), default=0)
+    _strs = []
+    for c in _comp:
+        if _comp[c].__doc__ is None:
+            doc = "No description provided"
+        else:
+            doc = " ".join(_comp[c].__doc__.split())
+        doc = justify(doc, 80, ml + 3)
+        _strs.append("{: >{}} : {}".format(c, ml, doc))
+    return o + "\n".join(_strs) + "\n"
+
+
+def remove_completer(name: str):
+    """removes a completer from xonsh
+
+    Parameters
+    ----------
+    name:
+        NAME is a unique name of a completer (run "completer list" to see the current
+        completers in order)
+    """
+    err = None
+    if name not in xsh_session.completers:
+        err = f"The name {name} is not a registered completer function."
+    if err is None:
+        del xsh_session.completers[name]
+        return
+    else:
+        return None, err + "\n", 1
+
 #
 # man
 #
@@ -1035,10 +918,11 @@ def complete_skipper(cmd, line, start, end, ctx):
 pickle = _LazyModule.load('pickle', 'pickle')
 # amalgamated builtins
 # amalgamated subprocess
+# amalgamated typing
 # amalgamated xonsh.lazyasd
 # amalgamated xonsh.completers.tools
-OPTIONS = None
-OPTIONS_PATH = None
+OPTIONS: tp.Optional[tp.Dict[str, tp.Any]] = None
+OPTIONS_PATH: tp.Optional[str] = None
 
 
 @xl.lazyobject
@@ -1119,6 +1003,14 @@ def cd_in_command(line):
     return have_cd
 
 
+def _get_normalized_pstring_quote(s):
+    for pre, norm_pre in (("p", "p"), ("pr", "pr"), ("rp", "pr"), ("fp", "pf")):
+        for q in ('"', "'"):
+            if s.startswith(f"{pre}{q}"):
+                return norm_pre, q
+    return (None, None)
+
+
 def _path_from_partial_string(inp, pos=None):
     if pos is None:
         pos = len(inp)
@@ -1137,6 +1029,14 @@ def _path_from_partial_string(inp, pos=None):
             else:
                 return None
         string = partial[startix:endix]
+
+    # If 'pr'/'rp', treat as raw string, otherwise strip leading 'p'
+    pstring_pre = _get_normalized_pstring_quote(quote)[0]
+    if pstring_pre == "pr":
+        string = f"r{string[2:]}"
+    elif pstring_pre == "p":
+        string = string[1:]
+
     end = xt.RE_STRING_START.sub("", quote)
     _string = string
     if not _string.endswith(end):
@@ -1365,6 +1265,10 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     lprefix = len(prefix)
     if p is not None:
         lprefix = len(p[0])
+        # Compensate for 'p' if p-string variant
+        pstring_pre = _get_normalized_pstring_quote(p[2])[0]
+        if pstring_pre in ("pr", "p"):
+            lprefix += 1
         prefix = p[1]
         path_str_start = p[2]
         path_str_end = p[3]
@@ -1408,12 +1312,12 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
         ):
             if xt.levenshtein(prefix, s, threshold) < threshold:
                 paths.add(s)
-    if tilde in prefix:
-        home = os.path.expanduser(tilde)
-        paths = {s.replace(home, tilde) for s in paths}
     if cdpath and cd_in_command(line):
         _add_cdpaths(paths, prefix)
     paths = set(filter(filtfunc, paths))
+    if tilde in prefix:
+        home = os.path.expanduser(tilde)
+        paths = {s.replace(home, tilde) for s in paths}
     paths, _ = _quote_paths(
         {_normpath(s) for s in paths}, path_str_start, path_str_end, append_end, cdpath
     )
@@ -1730,6 +1634,122 @@ def complete_import(prefix, line, start, end, ctx):
 def complete_module(prefix):
     return {s for s in sys.modules if get_filter_function()(s, prefix)}
 
+#
+# _aliases
+#
+ap = _LazyModule.load('argparse', 'argparse', 'ap')
+# amalgamated builtins
+xcli = _LazyModule.load('xonsh', 'xonsh.cli_utils', 'xcli')
+# amalgamated xonsh.lazyasd
+# amalgamated xonsh.completers.completer
+_add_one_completer = add_one_completer
+
+
+def _remove_completer(args):
+    """for backward compatibility"""
+    return remove_completer(args[0])
+
+
+def _register_completer(name: str, func: str, pos="start", stack=None):
+    """adds a new completer to xonsh
+
+    Parameters
+    ----------
+    name
+        unique name to use in the listing (run "completer list" to see the
+        current completers in order)
+
+    func
+        the name of a completer function to use.  This should be a function
+         of the following arguments, and should return a set of valid completions
+         for the given prefix.  If this completer should not be used in a given
+         context, it should return an empty set or None.
+
+         Arguments to FUNC:
+           * prefix: the string to be matched
+           * line: a string representing the whole current line, for context
+           * begidx: the index at which prefix starts in line
+           * endidx: the index at which prefix ends in line
+           * ctx: the current Python environment
+
+         If the completer expands the prefix in any way, it should return a tuple
+         of two elements: the first should be the set of completions, and the
+         second should be the length of the modified prefix (for an example, see
+         xonsh.completers.path.complete_path).
+
+    pos
+        position into the list of completers at which the new
+        completer should be added.  It can be one of the following values:
+        * "start" indicates that the completer should be added to the start of
+                 the list of completers (it should be run before all others)
+        * "end" indicates that the completer should be added to the end of the
+               list of completers (it should be run after all others)
+        * ">KEY", where KEY is a pre-existing name, indicates that this should
+                 be added after the completer named KEY
+        * "<KEY", where KEY is a pre-existing name, indicates that this should
+                 be added before the completer named KEY
+        (Default value: "start")
+    """
+    err = None
+    func_name = func
+    xsh = builtins.__xonsh__  # type: ignore
+    if name in xsh.completers:
+        err = f"The name {name} is already a registered completer function."
+    else:
+        if func_name in xsh.ctx:
+            func = xsh.ctx[func_name]
+            if not callable(func):
+                err = f"{func_name} is not callable"
+        else:
+            for frame_info in stack:
+                frame = frame_info[0]
+                if func_name in frame.f_locals:
+                    func = frame.f_locals[func_name]
+                    break
+                elif func_name in frame.f_globals:
+                    func = frame.f_globals[func_name]
+                    break
+            else:
+                err = "No such function: %s" % func_name
+    if err is None:
+        _add_one_completer(name, func, pos)
+    else:
+        return None, err + "\n", 1
+
+
+@xl.lazyobject
+def _parser() -> ap.ArgumentParser:
+    parser = xcli.make_parser(completer_alias)
+    commands = parser.add_subparsers(title="commands")
+
+    xcli.make_parser(
+        _register_completer,
+        commands,
+        params={
+            "name": {},
+            "func": {},
+            "pos": {"default": "start", "nargs": "?"},
+        },
+        prog="add",
+    )
+
+    xcli.make_parser(
+        remove_completer,
+        commands,
+        params={"name": {}},
+        prog="remove",
+    )
+
+    xcli.make_parser(list_completers, commands, prog="list")
+    return parser
+
+
+def completer_alias(args, stdin=None, stdout=None, stderr=None, spec=None, stack=None):
+    """CLI to add/remove/list xonsh auto-complete functions"""
+    ns = _parser.parse_args(args)
+    kwargs = vars(ns)
+    return xcli.dispatch(**kwargs, stdin=stdin, stdout=stdout, stack=stack)
+
 #
 # base
 #
@@ -1800,7 +1820,10 @@ def complete_cd(prefix, line, start, end, ctx):
     Completion for "cd", includes only valid directory names.
     """
     if start != 0 and line.split(" ")[0] == "cd":
-        return complete_dir(prefix, line, start, end, ctx, True)
+        results, prefix = complete_dir(prefix, line, start, end, ctx, True)
+        if len(results) == 0:
+            raise StopIteration
+        return results, prefix
     return set()
 
 
@@ -1815,6 +1838,8 @@ def complete_rmdir(prefix, line, start, end, ctx):
             if i.startswith(prefix)
         }
         comps, lp = complete_dir(prefix, line, start, end, ctx, True)
+        if len(comps) == 0 and len(opts) == 0:
+            raise StopIteration
         return comps | opts, lp
     return set()
 
