@@ -248,11 +248,15 @@ class DummyHistory(History):
 os = _LazyModule.load('os', 'os')
 sys = _LazyModule.load('sys', 'sys')
 time = _LazyModule.load('time', 'time')
-json = _LazyModule.load('json', 'json')
 builtins = _LazyModule.load('builtins', 'builtins')
 # amalgamated collections
 threading = _LazyModule.load('threading', 'threading')
 cabc = _LazyModule.load('collections', 'collections.abc', 'cabc')
+try:
+    import ujson as json
+except ImportError:
+    import json  # type: ignore
+
 # amalgamated xonsh.history.base
 xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
 xlj = _LazyModule.load('xonsh', 'xonsh.lazyjson', 'xlj')
@@ -422,6 +426,7 @@ class JsonHistoryGC(threading.Thread):
         boot = uptime.boottime()
         fs = _xhj_get_history_files(sort=False)
         files = []
+        time_start = time.time()
         for f in fs:
             try:
                 cur_file_size = os.path.getsize(f)
@@ -448,7 +453,12 @@ class JsonHistoryGC(threading.Thread):
                 )
                 lj.close()
                 if xonsh_debug:
-                    print(f"... Enumerated {len(files):7d} history files.\r", end="")
+                    time_lag = time.time() - time_start
+                    print(
+                        f"[history.{json.__name__}] Enumerated {len(files):,d} history files for {time_lag:0.4f}s.\r",
+                        end="",
+                        file=sys.stderr,
+                    )
             except (IOError, OSError, ValueError):
                 continue
         files.sort()  # this sorts by elements of the tuple,
@@ -504,6 +514,7 @@ class JsonHistoryFlusher(threading.Thread):
                 if self.skip is not None:
                     self.skip(1)
                 continue
+
             cmds.append(cmd)
             last_inp = cmd["inp"]
         with open(self.filename, "r", newline="\n") as f:
@@ -622,10 +633,17 @@ class JsonHistory(History):
         self._skipped = 0
         self.last_cmd_out = None
         self.last_cmd_rtn = None
+
         meta["cmds"] = []
         meta["sessionid"] = str(self.sessionid)
         with open(self.filename, "w", newline="\n") as f:
             xlj.ljdump(meta, f, sort_keys=True)
+
+        try:
+            os.chmod(self.filename, 0o600)
+        except Exception:  # pylint: disable=broad-except
+            pass
+
         self.gc = JsonHistoryGC() if gc else None
         # command fields that are known
         self.tss = JsonCommandField("ts", self)
@@ -646,6 +664,8 @@ class JsonHistory(History):
             added to the history list. It should contain the keys ``inp``,
             ``rtn`` and ``ts``. These key names mirror the same names defined
             as instance variables in the ``HistoryEntry`` class.
+            Additionally, an optional key ``spc`` may be present which will
+            affect commands from being stored if ignorespace is in $HISTCONTROL.
 
         Returns
         -------
@@ -654,8 +674,20 @@ class JsonHistory(History):
         """
         if not self.remember_history:
             return
+
+        opts = builtins.__xonsh__.env.get("HISTCONTROL")
+        skipped_by_ignore_space = "ignorespace" in opts and cmd.get("spc")
+        if skipped_by_ignore_space:
+            return None
+
         self.buffer.append(cmd)
         self._len += 1  # must come before flushing
+
+        try:
+            del cmd["spc"]
+        except KeyError:
+            pass
+
         if len(self.buffer) >= self.buffersize:
             hf = self.flush()
         else:
@@ -719,7 +751,7 @@ class JsonHistory(History):
                 continue
             try:
                 commands = json_file.load()["cmds"]
-            except json.decoder.JSONDecodeError:
+            except (json.decoder.JSONDecodeError, ValueError):
                 # file is corrupted somehow
                 if builtins.__xonsh__.env.get("XONSH_DEBUG") > 0:
                     msg = "xonsh history file {0!r} is not valid JSON"
@@ -773,7 +805,7 @@ class JsonHistory(History):
 """Implements the xonsh history backend via sqlite3."""
 # amalgamated builtins
 # amalgamated collections
-# amalgamated json
+json = _LazyModule.load('json', 'json')
 # amalgamated os
 sqlite3 = _LazyModule.load('sqlite3', 'sqlite3')
 # amalgamated sys
@@ -836,6 +868,13 @@ def _xh_sqlite_create_history_table(cursor):
         except sqlite3.OperationalError:
             pass
 
+        # add index on inp. since we query when erasedups is True
+        cursor.execute(
+            f"""\
+CREATE INDEX IF NOT EXISTS  idx_inp_history
+ON {XH_SQLITE_TABLE_NAME}(inp);"""
+        )
+
         # mark that this function ran for this session
         setattr(XH_SQLITE_CACHE, XH_SQLITE_CREATED_SQL_TBL, True)
 
@@ -955,6 +994,15 @@ def xh_sqlite_delete_items(size_to_keep, filename=None):
         return _xh_sqlite_delete_records(c, size_to_keep)
 
 
+def xh_sqlite_wipe_session(sessionid=None, filename=None):
+    """Wipe the current session's entries from the database."""
+    sql = "DELETE FROM xonsh_history WHERE sessionid = ?"
+    with _xh_sqlite_get_conn(filename=filename) as conn:
+        c = conn.cursor()
+        _xh_sqlite_create_history_table(c)
+        c.execute(sql, (str(sessionid),))
+
+
 class SqliteHistoryGC(threading.Thread):
     """Shell history garbage collection."""
 
@@ -1005,6 +1053,15 @@ class SqliteHistory(History):
         self.outs = []
         self.tss = []
 
+        if not os.path.exists(self.filename):
+            with _xh_sqlite_get_conn(filename=self.filename) as conn:
+                if conn:
+                    pass
+            try:
+                os.chmod(self.filename, 0o600)
+            except Exception:  # pylint: disable=broad-except
+                pass
+
         # during init rerun create command
         setattr(XH_SQLITE_CACHE, XH_SQLITE_CREATED_SQL_TBL, False)
 
@@ -1014,11 +1071,7 @@ class SqliteHistory(History):
         envs = builtins.__xonsh__.env
         inp = cmd["inp"].rstrip()
         self.inps.append(inp)
-        store_stdout = envs.get("XONSH_STORE_STDOUT", False)
-        if store_stdout:
-            self.outs.append(cmd.get("out"))
-        else:
-            self.outs.append(None)
+        self.outs.append(cmd.get("out"))
         self.rtns.append(cmd["rtn"])
         self.tss.append(cmd.get("ts", (None, None)))
 
@@ -1029,14 +1082,25 @@ class SqliteHistory(History):
         if "ignoreerr" in opts and cmd["rtn"] != 0:
             # Skipping failed cmd
             return
+        if "ignorespace" in opts and cmd.get("spc"):
+            # Skipping cmd starting with space
+            return
+
+        try:
+            del cmd["spc"]
+        except KeyError:
+            pass
         self._last_hist_inp = inp
-        xh_sqlite_append_history(
-            cmd,
-            str(self.sessionid),
-            store_stdout,
-            filename=self.filename,
-            remove_duplicates=("erasedups" in opts),
-        )
+        try:
+            xh_sqlite_append_history(
+                cmd,
+                str(self.sessionid),
+                store_stdout=envs.get("XONSH_STORE_STDOUT", False),
+                filename=self.filename,
+                remove_duplicates=("erasedups" in opts),
+            )
+        except sqlite3.OperationalError as err:
+            print(f"SQLite History Backend Error: {err}")
 
     def all_items(self, newest_first=False, session_id=None):
         """Display all history items."""
@@ -1076,12 +1140,7 @@ class SqliteHistory(History):
         self.outs = []
         self.tss = []
 
-        # Wipe the current session's entries from the database.
-        sql = "DELETE FROM xonsh_history WHERE sessionid = ?"
-        with _xh_sqlite_get_conn(filename=self.filename) as conn:
-            c = conn.cursor()
-            _xh_sqlite_create_history_table(c)
-            c.execute(sql, (str(self.sessionid),))
+        xh_sqlite_wipe_session(sessionid=self.sessionid, filename=self.filename)
 
 #
 # main
