@@ -3,12 +3,16 @@
 import os
 import sys
 import time
-import json
 import builtins
 import collections
 import threading
 import collections.abc as cabc
 
+try:
+    import ujson as json
+except ImportError:
+    import json  # type: ignore
+
 from xonsh.history.base import History
 import xonsh.tools as xt
 import xonsh.lazyjson as xlj
@@ -180,6 +184,7 @@ class JsonHistoryGC(threading.Thread):
         boot = uptime.boottime()
         fs = _xhj_get_history_files(sort=False)
         files = []
+        time_start = time.time()
         for f in fs:
             try:
                 cur_file_size = os.path.getsize(f)
@@ -206,7 +211,12 @@ class JsonHistoryGC(threading.Thread):
                 )
                 lj.close()
                 if xonsh_debug:
-                    print(f"... Enumerated {len(files):7d} history files.\r", end="")
+                    time_lag = time.time() - time_start
+                    print(
+                        f"[history.{json.__name__}] Enumerated {len(files):,d} history files for {time_lag:0.4f}s.\r",
+                        end="",
+                        file=sys.stderr,
+                    )
             except (IOError, OSError, ValueError):
                 continue
         files.sort()  # this sorts by elements of the tuple,
@@ -262,6 +272,7 @@ class JsonHistoryFlusher(threading.Thread):
                 if self.skip is not None:
                     self.skip(1)
                 continue
+
             cmds.append(cmd)
             last_inp = cmd["inp"]
         with open(self.filename, "r", newline="\n") as f:
@@ -380,10 +391,17 @@ class JsonHistory(History):
         self._skipped = 0
         self.last_cmd_out = None
         self.last_cmd_rtn = None
+
         meta["cmds"] = []
         meta["sessionid"] = str(self.sessionid)
         with open(self.filename, "w", newline="\n") as f:
             xlj.ljdump(meta, f, sort_keys=True)
+
+        try:
+            os.chmod(self.filename, 0o600)
+        except Exception:  # pylint: disable=broad-except
+            pass
+
         self.gc = JsonHistoryGC() if gc else None
         # command fields that are known
         self.tss = JsonCommandField("ts", self)
@@ -404,6 +422,8 @@ class JsonHistory(History):
             added to the history list. It should contain the keys ``inp``,
             ``rtn`` and ``ts``. These key names mirror the same names defined
             as instance variables in the ``HistoryEntry`` class.
+            Additionally, an optional key ``spc`` may be present which will
+            affect commands from being stored if ignorespace is in $HISTCONTROL.
 
         Returns
         -------
@@ -412,8 +432,20 @@ class JsonHistory(History):
         """
         if not self.remember_history:
             return
+
+        opts = builtins.__xonsh__.env.get("HISTCONTROL")
+        skipped_by_ignore_space = "ignorespace" in opts and cmd.get("spc")
+        if skipped_by_ignore_space:
+            return None
+
         self.buffer.append(cmd)
         self._len += 1  # must come before flushing
+
+        try:
+            del cmd["spc"]
+        except KeyError:
+            pass
+
         if len(self.buffer) >= self.buffersize:
             hf = self.flush()
         else:
@@ -477,7 +509,7 @@ class JsonHistory(History):
                 continue
             try:
                 commands = json_file.load()["cmds"]
-            except json.decoder.JSONDecodeError:
+            except (json.decoder.JSONDecodeError, ValueError):
                 # file is corrupted somehow
                 if builtins.__xonsh__.env.get("XONSH_DEBUG") > 0:
                     msg = "xonsh history file {0!r} is not valid JSON"
