@@ -67,6 +67,13 @@ def _xh_sqlite_create_history_table(cursor):
         except sqlite3.OperationalError:
             pass
 
+        # add index on inp. since we query when erasedups is True
+        cursor.execute(
+            f"""\
+CREATE INDEX IF NOT EXISTS  idx_inp_history
+ON {XH_SQLITE_TABLE_NAME}(inp);"""
+        )
+
         # mark that this function ran for this session
         setattr(XH_SQLITE_CACHE, XH_SQLITE_CREATED_SQL_TBL, True)
 
@@ -186,6 +193,15 @@ def xh_sqlite_delete_items(size_to_keep, filename=None):
         return _xh_sqlite_delete_records(c, size_to_keep)
 
 
+def xh_sqlite_wipe_session(sessionid=None, filename=None):
+    """Wipe the current session's entries from the database."""
+    sql = "DELETE FROM xonsh_history WHERE sessionid = ?"
+    with _xh_sqlite_get_conn(filename=filename) as conn:
+        c = conn.cursor()
+        _xh_sqlite_create_history_table(c)
+        c.execute(sql, (str(sessionid),))
+
+
 class SqliteHistoryGC(threading.Thread):
     """Shell history garbage collection."""
 
@@ -236,6 +252,15 @@ class SqliteHistory(History):
         self.outs = []
         self.tss = []
 
+        if not os.path.exists(self.filename):
+            with _xh_sqlite_get_conn(filename=self.filename) as conn:
+                if conn:
+                    pass
+            try:
+                os.chmod(self.filename, 0o600)
+            except Exception:  # pylint: disable=broad-except
+                pass
+
         # during init rerun create command
         setattr(XH_SQLITE_CACHE, XH_SQLITE_CREATED_SQL_TBL, False)
 
@@ -245,11 +270,7 @@ class SqliteHistory(History):
         envs = builtins.__xonsh__.env
         inp = cmd["inp"].rstrip()
         self.inps.append(inp)
-        store_stdout = envs.get("XONSH_STORE_STDOUT", False)
-        if store_stdout:
-            self.outs.append(cmd.get("out"))
-        else:
-            self.outs.append(None)
+        self.outs.append(cmd.get("out"))
         self.rtns.append(cmd["rtn"])
         self.tss.append(cmd.get("ts", (None, None)))
 
@@ -260,14 +281,25 @@ class SqliteHistory(History):
         if "ignoreerr" in opts and cmd["rtn"] != 0:
             # Skipping failed cmd
             return
+        if "ignorespace" in opts and cmd.get("spc"):
+            # Skipping cmd starting with space
+            return
+
+        try:
+            del cmd["spc"]
+        except KeyError:
+            pass
         self._last_hist_inp = inp
-        xh_sqlite_append_history(
-            cmd,
-            str(self.sessionid),
-            store_stdout,
-            filename=self.filename,
-            remove_duplicates=("erasedups" in opts),
-        )
+        try:
+            xh_sqlite_append_history(
+                cmd,
+                str(self.sessionid),
+                store_stdout=envs.get("XONSH_STORE_STDOUT", False),
+                filename=self.filename,
+                remove_duplicates=("erasedups" in opts),
+            )
+        except sqlite3.OperationalError as err:
+            print(f"SQLite History Backend Error: {err}")
 
     def all_items(self, newest_first=False, session_id=None):
         """Display all history items."""
@@ -307,9 +339,4 @@ class SqliteHistory(History):
         self.outs = []
         self.tss = []
 
-        # Wipe the current session's entries from the database.
-        sql = "DELETE FROM xonsh_history WHERE sessionid = ?"
-        with _xh_sqlite_get_conn(filename=self.filename) as conn:
-            c = conn.cursor()
-            _xh_sqlite_create_history_table(c)
-            c.execute(sql, (str(self.sessionid),))
+        xh_sqlite_wipe_session(sessionid=self.sessionid, filename=self.filename)
