@@ -4,14 +4,13 @@ on a platform.
 """
 import os
 import sys
-import ctypes
+import ctypes  # noqa
 import signal
 import pathlib
 import builtins
 import platform
 import functools
 import subprocess
-import collections
 import collections.abc as cabc
 import importlib.util
 
@@ -169,13 +168,13 @@ def ptk_version_info():
         return None
 
 
-minimum_required_ptk_version = (2, 0)
+minimum_required_ptk_version = (2, 0, 0)
 """Minimum version of prompt-toolkit supported by Xonsh"""
 
 
 @functools.lru_cache(1)
 def ptk_above_min_supported():
-    return ptk_version_info() and ptk_version_info()[:2] >= minimum_required_ptk_version
+    return ptk_version_info() and ptk_version_info() >= minimum_required_ptk_version
 
 
 @functools.lru_cache(1)
@@ -200,10 +199,9 @@ def ptk_below_max_supported():
 def best_shell_type():
     if builtins.__xonsh__.env.get("TERM", "") == "dumb":
         return "dumb"
-    elif ON_WINDOWS or has_prompt_toolkit():
+    if has_prompt_toolkit():
         return "prompt_toolkit"
-    else:
-        return "readline"
+    return "readline"
 
 
 @functools.lru_cache(1)
@@ -305,42 +303,6 @@ DEFAULT_ENCODING = sys.getdefaultencoding()
 """ Default string encoding. """
 
 
-if PYTHON_VERSION_INFO < (3, 5, 0):
-
-    class DirEntry:
-        def __init__(self, directory, name):
-            self.__path__ = pathlib.Path(directory) / name
-            self.name = name
-            self.path = str(self.__path__)
-            self.is_symlink = self.__path__.is_symlink
-
-        def inode(self):
-            return os.stat(self.path, follow_symlinks=False).st_ino
-
-        def is_dir(self, *, follow_symlinks=True):
-            if follow_symlinks:
-                return self.__path__.is_dir()
-            else:
-                return not self.__path__.is_symlink() and self.__path__.is_dir()
-
-        def is_file(self, *, follow_symlinks=True):
-            if follow_symlinks:
-                return self.__path__.is_file()
-            else:
-                return not self.__path__.is_symlink() and self.__path__.is_file()
-
-        def stat(self, *, follow_symlinks=True):
-            return os.stat(self.path, follow_symlinks=follow_symlinks)
-
-    def scandir(path):
-        """ Compatibility layer for  `os.scandir` from Python 3.5+. """
-        return (DirEntry(path, x) for x in os.listdir(path))
-
-
-else:
-    scandir = os.scandir
-
-
 #
 # Linux distro
 #
@@ -423,10 +385,10 @@ def windows_bash_command():
 if ON_WINDOWS:
 
     class OSEnvironCasePreserving(cabc.MutableMapping):
-        """ Case-preserving wrapper for os.environ on Windows.
-            It uses nt.environ to get the correct cased keys on
-            initialization. It also preserves the case of any variables
-            add after initialization.
+        """Case-preserving wrapper for os.environ on Windows.
+        It uses nt.environ to get the correct cased keys on
+        initialization. It also preserves the case of any variables
+        add after initialization.
         """
 
         def __init__(self):
@@ -435,8 +397,8 @@ if ON_WINDOWS:
             self._upperkeys = dict((k.upper(), k) for k in nt.environ)
 
         def _sync(self):
-            """ Ensure that the case sensitive map of the keys are
-                in sync with os.environ
+            """Ensure that the case sensitive map of the keys are
+            in sync with os.environ
             """
             envkeys = set(os.environ.keys())
             for key in envkeys.difference(self._upperkeys):
@@ -580,6 +542,11 @@ def LIBC():
         libc = ctypes.CDLL("cygwin1.dll")
     elif ON_MSYS:
         libc = ctypes.CDLL("msys-2.0.dll")
+    elif ON_FREEBSD:
+        try:
+            libc = ctypes.CDLL("libc.so.7")
+        except OSError:
+            libc = None
     elif ON_BSD:
         try:
             libc = ctypes.CDLL("libc.so")
