@@ -90,10 +90,10 @@ def _replace_home_cwd():
 def _collapsed_pwd():
     sep = xt.get_sep()
     pwd = _replace_home_cwd().split(sep)
-    l = len(pwd)
-    leader = sep if l > 0 and len(pwd[0]) == 0 else ""
+    size = len(pwd)
+    leader = sep if size > 0 and len(pwd[0]) == 0 else ""
     base = [
-        i[0] if ix != l - 1 and i[0] != "." else i[0:2] if ix != l - 1 else i
+        i[0] if ix != size - 1 and i[0] != "." else i[0:2] if ix != size - 1 else i
         for ix, i in enumerate(pwd)
         if len(i) > 0
     ]
@@ -396,13 +396,13 @@ def gitstatus_prompt():
         symbol = _get_def(category.upper())
         value = getattr(s, category)
         if symbol and value > 0:
-            ret += symbol + str(value) + "{NO_COLOR}"
+            ret += symbol + str(value) + "{RESET}"
     if s.staged + s.conflicts + s.changed + s.deleted + s.untracked + s.stashed == 0:
         symbol = _get_def("CLEAN")
         if symbol:
-            ret += symbol + "{NO_COLOR}"
+            ret += symbol + "{RESET}"
     ret = ret.rstrip("|")
-    ret += "{NO_COLOR}"
+    ret += "{RESET}"
 
     return ret
 
@@ -449,6 +449,7 @@ queue = _LazyModule.load('queue', 'queue')
 threading = _LazyModule.load('threading', 'threading')
 # amalgamated subprocess
 re = _LazyModule.load('re', 're')
+pathlib = _LazyModule.load('pathlib', 'pathlib')
 # amalgamated xonsh.tools
 from xonsh.lazyasd import LazyObject
 
@@ -459,29 +460,23 @@ RE_REMOVE_ANSI = LazyObject(
 )
 
 
+def _run_git_cmd(cmd):
+    # create a safe detyped env dictionary and update with the additional git environment variables
+    # when running git status commands we do not want to acquire locks running command like git status
+    denv = dict(builtins.__xonsh__.env.detype())
+    denv.update({"GIT_OPTIONAL_LOCKS": "0"})
+    return subprocess.check_output(cmd, env=denv, stderr=subprocess.DEVNULL)
+
+
 def _get_git_branch(q):
-    denv = builtins.__xonsh__.env.detype()
     try:
-        branches = xt.decode_bytes(
-            subprocess.check_output(
-                ["git", "branch"], env=denv, stderr=subprocess.DEVNULL
-            )
-        ).splitlines()
+        cmd = ["git", "rev-parse", "--abbrev-ref", "HEAD"]
+        branch = xt.decode_bytes(_run_git_cmd(cmd))
+        branch = branch.splitlines()[0] or None
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
     else:
-        for branch in branches:
-            if not branch.startswith("* "):
-                continue
-            elif branch.endswith(")"):
-                branch = branch.split()[-1][:-1]
-            else:
-                branch = branch.split()[-1]
-
-            q.put(branch)
-            break
-        else:
-            q.put(None)
+        q.put(branch)
 
 
 def get_git_branch():
@@ -497,7 +492,6 @@ def get_git_branch():
     t.join(timeout=timeout)
     try:
         branch = q.get_nowait()
-        # branch = RE_REMOVE_ANSI.sub("", branch or "")
         if branch:
             branch = RE_REMOVE_ANSI.sub("", branch)
     except queue.Empty:
@@ -511,7 +505,7 @@ def _get_hg_root(q):
         if not os.path.isdir(_curpwd):
             return False
         try:
-            dot_hg_is_in_curwd = any([b.name == ".hg" for b in xt.scandir(_curpwd)])
+            dot_hg_is_in_curwd = any([b.name == ".hg" for b in os.scandir(_curpwd)])
         except OSError:
             return False
         if dot_hg_is_in_curwd:
@@ -535,32 +529,34 @@ def get_hg_branch(root=None):
     t.start()
     t.join(timeout=timeout)
     try:
-        root = q.get_nowait()
+        root = pathlib.Path(q.get_nowait())
     except queue.Empty:
         return None
     if env.get("VC_HG_SHOW_BRANCH"):
         # get branch name
-        branch_path = os.path.sep.join([root, ".hg", "branch"])
-        if os.path.exists(branch_path):
+        branch_path = root / ".hg" / "branch"
+        if branch_path.exists():
             with open(branch_path, "r") as branch_file:
-                branch = branch_file.read()
+                branch = branch_file.read().strip()
         else:
             branch = "default"
     else:
         branch = ""
-    # add bookmark, if we can
-    bookmark_path = os.path.sep.join([root, ".hg", "bookmarks.current"])
-    if os.path.exists(bookmark_path):
-        with open(bookmark_path, "r") as bookmark_file:
-            active_bookmark = bookmark_file.read()
-        if env.get("VC_HG_SHOW_BRANCH") is True:
-            branch = "{0}, {1}".format(
-                *(b.strip(os.linesep) for b in (branch, active_bookmark))
-            )
-        else:
-            branch = active_bookmark.strip(os.linesep)
-    else:
-        branch = branch.strip(os.linesep)
+    # add activated bookmark and topic
+    for filename in ["bookmarks.current", "topic"]:
+        feature_branch_path = root / ".hg" / filename
+        if feature_branch_path.exists():
+            with open(feature_branch_path) as file:
+                feature_branch = file.read().strip()
+            if feature_branch:
+                if branch:
+                    if filename == "topic":
+                        branch = f"{branch}/{feature_branch}"
+                    else:
+                        branch = f"{branch}, {feature_branch}"
+                else:
+                    branch = feature_branch
+
     return branch
 
 
@@ -585,6 +581,15 @@ def _first_branch_timeout_message():
     )
 
 
+def _vc_has(binary):
+    """ This allows us to locate binaries after git only if necessary """
+    cmds = builtins.__xonsh__.commands_cache
+    if cmds.is_empty():
+        return bool(cmds.locate_binary(binary, ignore_alias=True))
+    else:
+        return bool(cmds.lazy_locate_binary(binary, ignore_alias=True))
+
+
 def current_branch():
     """Gets the branch for a current working directory. Returns an empty string
     if the cwd is not a repository.  This currently only works for git and hg
@@ -592,17 +597,9 @@ def current_branch():
     '<branch-timeout>' is returned.
     """
     branch = None
-    cmds = builtins.__xonsh__.commands_cache
-    # check for binary only once
-    if cmds.is_empty():
-        has_git = bool(cmds.locate_binary("git", ignore_alias=True))
-        has_hg = bool(cmds.locate_binary("hg", ignore_alias=True))
-    else:
-        has_git = bool(cmds.lazy_locate_binary("git", ignore_alias=True))
-        has_hg = bool(cmds.lazy_locate_binary("hg", ignore_alias=True))
-    if has_git:
+    if _vc_has("git"):
         branch = get_git_branch()
-    if not branch and has_hg:
+    if not branch and _vc_has("hg"):
         branch = get_hg_branch()
     if isinstance(branch, subprocess.TimeoutExpired):
         branch = "<branch-timeout>"
@@ -611,26 +608,28 @@ def current_branch():
 
 
 def _git_dirty_working_directory(q, include_untracked):
-    status = None
-    denv = builtins.__xonsh__.env.detype()
     try:
         cmd = ["git", "status", "--porcelain"]
         if include_untracked:
-            cmd.append("--untracked-files=normal")
+            cmd += ["--untracked-files=normal"]
         else:
-            cmd.append("--untracked-files=no")
-        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL, env=denv)
+            cmd += ["--untracked-files=no"]
+        status = _run_git_cmd(cmd)
+        if status is not None:
+            q.put(bool(status))
+        else:
+            q.put(None)
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
-    if status is not None:
-        return q.put(bool(status))
 
 
-def git_dirty_working_directory(include_untracked=False):
+def git_dirty_working_directory():
     """Returns whether or not the git directory is dirty. If this could not
     be determined (timeout, file not found, etc.) then this returns None.
     """
-    timeout = builtins.__xonsh__.env.get("VC_BRANCH_TIMEOUT")
+    env = builtins.__xonsh__.env
+    timeout = env.get("VC_BRANCH_TIMEOUT")
+    include_untracked = env.get("VC_GIT_INCLUDE_UNTRACKED")
     q = queue.Queue()
     t = threading.Thread(
         target=_git_dirty_working_directory, args=(q, include_untracked)
@@ -677,10 +676,9 @@ def dirty_working_directory():
     None. Currently supports git and hg.
     """
     dwd = None
-    cmds = builtins.__xonsh__.commands_cache
-    if cmds.lazy_locate_binary("git", ignore_alias=True):
+    if _vc_has("git"):
         dwd = git_dirty_working_directory()
-    if cmds.lazy_locate_binary("hg", ignore_alias=True) and dwd is None:
+    if dwd is None and _vc_has("hg"):
         dwd = hg_dirty_working_directory()
     return dwd
 
@@ -725,6 +723,7 @@ itertools = _LazyModule.load('itertools', 'itertools')
 # amalgamated re
 socket = _LazyModule.load('socket', 'socket')
 # amalgamated sys
+tp = _LazyModule.load('typing', 'typing', 'tp')
 # amalgamated xonsh.lazyasd
 # amalgamated xonsh.tools
 # amalgamated xonsh.platform
@@ -739,6 +738,54 @@ def DEFAULT_PROMPT():
     return default_prompt()
 
 
+class _ParsedToken(tp.NamedTuple):
+    """It can either be a literal value alone or a field and its resultant value"""
+
+    value: str
+    field: tp.Optional[str] = None
+
+
+class ParsedTokens(tp.NamedTuple):
+    tokens: tp.List[_ParsedToken]
+    template: tp.Union[str, tp.Callable]
+
+    def process(self) -> str:
+        """Wrapper that gets formatter-function from environment and returns final prompt."""
+        processor = builtins.__xonsh__.env.get(  # type: ignore
+            "PROMPT_TOKENS_FORMATTER", prompt_tokens_formatter_default
+        )
+        return processor(self)
+
+    def update(
+        self,
+        idx: int,
+        val: tp.Optional[str],
+        spec: tp.Optional[str],
+        conv: tp.Optional[str],
+    ) -> None:
+        """Update tokens list in-place"""
+        if idx < len(self.tokens):
+            tok = self.tokens[idx]
+            self.tokens[idx] = _ParsedToken(_format_value(val, spec, conv), tok.field)
+
+
+def prompt_tokens_formatter_default(container: ParsedTokens) -> str:
+    """
+        Join the tokens
+
+    Parameters
+    ----------
+    container: ParsedTokens
+        parsed tokens holder
+
+    Returns
+    -------
+    str
+        process the tokens and finally return the prompt string
+    """
+    return "".join([tok.value for tok in container.tokens])
+
+
 class PromptFormatter:
     """Class that holds all the related prompt formatting methods,
     uses the ``PROMPT_FIELDS`` envvar (no color formatting).
@@ -747,54 +794,67 @@ class PromptFormatter:
     def __init__(self):
         self.cache = {}
 
-    def __call__(self, template=DEFAULT_PROMPT, fields=None):
+    def __call__(self, template=DEFAULT_PROMPT, fields=None, **kwargs) -> str:
         """Formats a xonsh prompt template string."""
+
+        # keep cache only during building prompt
+        self.cache.clear()
+
         if fields is None:
-            self.fields = builtins.__xonsh__.env.get("PROMPT_FIELDS", PROMPT_FIELDS)
+            self.fields = builtins.__xonsh__.env.get("PROMPT_FIELDS", PROMPT_FIELDS)  # type: ignore
         else:
             self.fields = fields
         try:
-            prompt = self._format_prompt(template=template)
-        except Exception:
+            toks = self._format_prompt(template=template, **kwargs)
+            prompt = toks.process()
+        except Exception as ex:
+            # make it obvious why it has failed
+            print(
+                f"Failed to format prompt `{template}`-> {type(ex)}:{ex}",
+                file=sys.stderr,
+            )
             return _failover_template_format(template)
-        # keep cache only during building prompt
-        self.cache.clear()
         return prompt
 
-    def _format_prompt(self, template=DEFAULT_PROMPT):
-        template = template() if callable(template) else template
+    def _format_prompt(self, template=DEFAULT_PROMPT, **kwargs) -> ParsedTokens:
+        tmpl = template() if callable(template) else template
         toks = []
-        for literal, field, spec, conv in xt.FORMATTER.parse(template):
-            toks.append(literal)
-            entry = self._format_field(field, spec, conv)
+        for literal, field, spec, conv in xt.FORMATTER.parse(tmpl):
+            if literal:
+                toks.append(_ParsedToken(literal))
+            entry = self._format_field(field, spec, conv, idx=len(toks), **kwargs)
             if entry is not None:
-                toks.append(entry)
-        return "".join(toks)
+                toks.append(_ParsedToken(entry, field))
 
-    def _format_field(self, field, spec, conv):
+        return ParsedTokens(toks, template)
+
+    def _format_field(self, field, spec="", conv=None, **kwargs):
         if field is None:
             return
         elif field.startswith("$"):
             val = builtins.__xonsh__.env[field[1:]]
             return _format_value(val, spec, conv)
         elif field in self.fields:
-            val = self._get_field_value(field)
+            val = self._get_field_value(field, spec=spec, conv=conv, **kwargs)
             return _format_value(val, spec, conv)
         else:
             # color or unknown field, return as is
             return "{" + field + "}"
 
-    def _get_field_value(self, field):
+    def _get_field_value(self, field, **kwargs):
         field_value = self.fields[field]
         if field_value in self.cache:
             return self.cache[field_value]
+        return self._no_cache_field_value(field, field_value, **kwargs)
+
+    def _no_cache_field_value(self, field, field_value, **_):
         try:
             value = field_value() if callable(field_value) else field_value
             self.cache[field_value] = value
         except Exception:
             print("prompt: error: on field {!r}" "".format(field), file=sys.stderr)
             xt.print_exception()
-            value = "{{BACKGROUND_RED}}{{ERROR:{}}}{{NO_COLOR}}".format(field)
+            value = "{{BACKGROUND_RED}}{{ERROR:{}}}{{RESET}}".format(field)
         return value
 
 
@@ -805,7 +865,7 @@ def PROMPT_FIELDS():
         prompt_end="#" if xt.is_superuser() else "$",
         hostname=socket.gethostname().split(".", 1)[0],
         cwd=_dynamically_collapsed_pwd,
-        cwd_dir=lambda: os.path.dirname(_replace_home_cwd()),
+        cwd_dir=lambda: os.path.join(os.path.dirname(_replace_home_cwd()), ""),
         cwd_base=lambda: os.path.basename(_replace_home_cwd()),
         short_cwd=_collapsed_pwd,
         curr_branch=current_branch,
@@ -828,21 +888,21 @@ def default_prompt():
         dp = (
             "{env_name}"
             "{BOLD_GREEN}{user}@{hostname}"
-            "{BOLD_BLUE} {cwd} {prompt_end}{NO_COLOR} "
+            "{BOLD_BLUE} {cwd} {prompt_end}{RESET} "
         )
     elif xp.ON_WINDOWS and not xp.win_ansi_support():
         dp = (
             "{env_name}"
             "{BOLD_INTENSE_GREEN}{user}@{hostname}{BOLD_INTENSE_CYAN} "
-            "{cwd}{branch_color}{curr_branch: {}}{NO_COLOR} "
-            "{BOLD_INTENSE_CYAN}{prompt_end}{NO_COLOR} "
+            "{cwd}{branch_color}{curr_branch: {}}{RESET} "
+            "{BOLD_INTENSE_CYAN}{prompt_end}{RESET} "
         )
     else:
         dp = (
             "{env_name}"
             "{BOLD_GREEN}{user}@{hostname}{BOLD_BLUE} "
-            "{cwd}{branch_color}{curr_branch: {}}{NO_COLOR} "
-            "{BOLD_BLUE}{prompt_end}{NO_COLOR} "
+            "{cwd}{branch_color}{curr_branch: {}}{RESET} "
+            "{BOLD_BLUE}{prompt_end}{RESET} "
         )
     return dp
 
@@ -890,7 +950,7 @@ def multiline_prompt(curr=""):
             basetoks.append(("\001" + pre + "\002", post))
             baselen += len(post)
     if baselen == 0:
-        return xt.format_color("{NO_COLOR}" + tail, hide=True)
+        return xt.format_color("{RESET}" + tail, hide=True)
     toks = basetoks * (headlen // baselen)
     n = headlen % baselen
     count = 0
@@ -906,7 +966,7 @@ def multiline_prompt(curr=""):
         count = newcount
         if n <= count:
             break
-    toks.append((xt.format_color("{NO_COLOR}", hide=True), tail))
+    toks.append((xt.format_color("{RESET}", hide=True), tail))
     rtn = "".join(itertools.chain.from_iterable(toks))
     return rtn
 
