@@ -9,7 +9,6 @@ import warnings
 from xonsh.platform import (
     best_shell_type,
     ptk_above_min_supported,
-    use_vended_prompt_toolkit,
     has_prompt_toolkit,
     minimum_required_ptk_version,
 )
@@ -57,15 +56,25 @@ Parameters:
 """,
 )
 
+events.doc(
+    "on_pre_prompt_format",
+    """
+on_pre_prompt_format() -> None
+
+Fires before the prompt will be formatted
+""",
+)
+
 events.doc(
     "on_pre_prompt",
     """
 on_pre_prompt() -> None
 
-Fires just before the prompt is shown
+Fires just before showing the prompt
 """,
 )
 
+
 events.doc(
     "on_post_prompt",
     """
@@ -160,15 +169,18 @@ class Shell(object):
             shell_type = simple_random_choice(("readline", "prompt_toolkit"))
         if shell_type == "prompt_toolkit":
             if not has_prompt_toolkit():
-                use_vended_prompt_toolkit()
+                warnings.warn(
+                    "'prompt-toolkit' python package is not installed. Falling back to readline."
+                )
+                shell_type = "readline"
             elif not ptk_above_min_supported():
                 warnings.warn(
                     "Installed prompt-toolkit version < v{}.{}.{} is not ".format(
                         *minimum_required_ptk_version
                     )
-                    + "supported. Falling back to the builtin prompt-toolkit."
+                    + "supported. Falling back to readline."
                 )
-                use_vended_prompt_toolkit()
+                shell_type = "readline"
             if init_shell_type in ("ptk1", "prompt_toolkit1"):
                 warnings.warn(
                     "$SHELL_TYPE='{}' now deprecated, please update your run control file'".format(
