@@ -47,7 +47,8 @@ from xonsh.tools import (
     color_style,
 )
 from xonsh.foreign_shells import CANON_SHELL_NAMES
-from xonsh.xontribs import xontrib_metadata, find_xontrib
+from xonsh.xontribs import find_xontrib, xontribs_loaded
+from xonsh.xontribs_meta import get_xontribs, Xontrib
 from xonsh.lazyasd import lazyobject
 
 HR = "'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'"
@@ -300,7 +301,7 @@ def make_envvar(name):
     """Makes a StoreNonEmpty node for an environment variable."""
     env = builtins.__xonsh__.env
     vd = env.get_docs(name)
-    if not vd.doc_configurable:
+    if not vd.is_configurable:
         return
     default = vd.doc_default
     if "\n" in default:
@@ -326,7 +327,7 @@ def make_envvar(name):
         show_conversion=True,
         path=path,
         retry=True,
-        store_raw=vd.doc_store_as_str,
+        store_raw=vd.can_store_as_str,
     )
     return mnode, pnode
 
@@ -356,25 +357,24 @@ def _xontrib_path(visitor=None, node=None, val=None):
     return ("xontribs", len(visitor.state.get("xontribs", ())))
 
 
-def make_xontrib(xontrib, package):
+def make_xontrib(xon_item: tp.Tuple[str, Xontrib]):
     """Makes a message and StoreNonEmpty node for a xontrib."""
-    name = xontrib.get("name", "<unknown-xontrib-name>")
+    name, xontrib = xon_item
+    name = name or "<unknown-xontrib-name>"
     msg = "\n{BOLD_CYAN}" + name + "{RESET}\n"
-    if "url" in xontrib:
-        msg += "{RED}url:{RESET} " + xontrib["url"] + "\n"
-    if "package" in xontrib:
-        msg += "{RED}package:{RESET} " + xontrib["package"] + "\n"
-    if "url" in package:
-        if "url" in xontrib and package["url"] != xontrib["url"]:
-            msg += "{RED}package-url:{RESET} " + package["url"] + "\n"
-    if "license" in package:
-        msg += "{RED}license:{RESET} " + package["license"] + "\n"
+    if xontrib.url:
+        msg += "{RED}url:{RESET} " + xontrib.url + "\n"
+    if xontrib.package:
+        pkg = xontrib.package
+        msg += "{RED}package:{RESET} " + pkg.name + "\n"
+        if pkg.url:
+            if xontrib.url and pkg.url != xontrib.url:
+                msg += "{RED}package-url:{RESET} " + pkg.url + "\n"
+        if pkg.license:
+            msg += "{RED}license:{RESET} " + pkg.license + "\n"
     msg += "{PURPLE}installed?{RESET} "
     msg += ("no" if find_xontrib(name) is None else "yes") + "\n"
-    desc = xontrib.get("description", "")
-    if not isinstance(desc, str):
-        desc = "".join(desc)
-    msg += _wrap_paragraphs(desc, width=69)
+    msg += _wrap_paragraphs(xontrib.description, width=69)
     if msg.endswith("\n"):
         msg = msg[:-1]
     mnode = wiz.Message(message=msg)
@@ -385,10 +385,7 @@ def make_xontrib(xontrib, package):
 
 def make_xontribs_wiz():
     """Makes a xontrib wizard."""
-    md = xontrib_metadata()
-    pkgs = [md["packages"].get(d.get("package", None), {}) for d in md["xontribs"]]
-    w = _make_flat_wiz(make_xontrib, md["xontribs"], pkgs)
-    return w
+    return _make_flat_wiz(make_xontrib, get_xontribs().items())
 
 
 def make_xonfig_wizard(default_file=None, confirm=False, no_wizard_file=None):
@@ -478,12 +475,22 @@ def _xonfig_format_human(data):
     wcol1 = wcol2 = 0
     for key, val in data:
         wcol1 = max(wcol1, len(key))
-        wcol2 = max(wcol2, len(str(val)))
+        if isinstance(val, list):
+            for subval in val:
+                wcol2 = max(wcol2, len(str(subval)))
+        else:
+            wcol2 = max(wcol2, len(str(val)))
     hr = "+" + ("-" * (wcol1 + 2)) + "+" + ("-" * (wcol2 + 2)) + "+\n"
     row = "| {key!s:<{wcol1}} | {val!s:<{wcol2}} |\n"
     s = hr
     for key, val in data:
-        s += row.format(key=key, wcol1=wcol1, val=val, wcol2=wcol2)
+        if isinstance(val, list) and val:
+            for i, subval in enumerate(val):
+                s += row.format(
+                    key=f"{key} {i+1}", wcol1=wcol1, val=subval, wcol2=wcol2
+                )
+        else:
+            s += row.format(key=key, wcol1=wcol1, val=val, wcol2=wcol2)
     s += hr
     return s
 
@@ -537,6 +544,8 @@ def _info(ns):
         pass
     data.extend([("on jupyter", jup_ksm is not None), ("jupyter kernel", jup_kernel)])
 
+    data.extend([("xontrib", xontribs_loaded())])
+
     formatter = _xonfig_format_json if ns.json else _xonfig_format_human
     s = formatter(data)
     return s
@@ -851,12 +860,18 @@ WELCOME_MSG = [
     ("{{INTENSE_RED}}~{{RESET}} {tagline} {{INTENSE_RED}}~{{RESET}}", "^", " "),
     "",
     ("{{INTENSE_BLACK}}", "<", "-"),
-    "{{GREEN}}xonfig{{RESET}} tutorial    {{INTENSE_WHITE}}->    Launch the tutorial in "
-    "the browser{{RESET}}",
-    "{{GREEN}}xonfig{{RESET}} web         {{INTENSE_WHITE}}->    Run the configuration "
-    "tool in the browser and claim your shell {{RESET}}",
-    "{{INTENSE_BLACK}}(Note: Run the configuration tool or create a "
-    "{{RED}}~/.xonshrc{{INTENSE_BLACK}} file to suppress the welcome screen)",
+    "",
+    (
+        "{{INTENSE_BLACK}}Create ~/.xonshrc file manually or use xonfig to suppress the welcome screen",
+        "^",
+        " ",
+    ),
+    "",
+    "{{INTENSE_BLACK}}Start from commands:",
+    "  {{GREEN}}xonfig{{RESET}} web         {{INTENSE_BLACK}}# Run the configuration tool in the browser to create ~/.xonshrc {{RESET}}",
+    "  {{GREEN}}xonfig{{RESET}} tutorial    {{INTENSE_BLACK}}# Open the xonsh tutorial in the browser{{RESET}}",
+    "",
+    ("{{INTENSE_BLACK}}", "<", "-"),
     "",
 ]
 
