@@ -6,7 +6,7 @@ import functools
 
 from xonsh.xoreutils import _which
 import xonsh.platform as xp
-import xonsh.proc as xproc
+import xonsh.procs.pipelines as xpp
 
 
 @functools.lru_cache()
@@ -34,7 +34,7 @@ def _which_create_parser():
         "-V",
         "--version",
         action="version",
-        version="{}".format(_which.__version__),
+        version=f"{_which.__version__}",
         help="Display the version of the python which module " "used by xonsh",
     )
     parser.add_argument(
@@ -85,12 +85,12 @@ def print_path(abs_name, from_where, stdout, verbose=False, captured=False):
         # Use list dir to get correct case for the filename
         # i.e. windows is case insensitive but case preserving
         p, f = os.path.split(abs_name)
-        f = next(s.name for s in xp.scandir(p) if s.name.lower() == f.lower())
+        f = next(s.name for s in os.scandir(p) if s.name.lower() == f.lower())
         abs_name = os.path.join(p, f)
         if builtins.__xonsh__.env.get("FORCE_POSIX_PATHS", False):
             abs_name.replace(os.sep, os.altsep)
     if verbose:
-        print("{} ({})".format(abs_name, from_where), file=stdout)
+        print(f"{abs_name} ({from_where})", file=stdout)
     else:
         end = "" if captured else "\n"
         print(abs_name, end=end, file=stdout)
@@ -104,7 +104,11 @@ def print_alias(arg, stdout, verbose=False):
         else:
             print(arg, file=stdout)
     else:
-        print("aliases['{}'] = {}".format(arg, builtins.aliases[arg]), file=stdout)
+        print(
+            "aliases['{}'] = {}".format(arg, builtins.aliases[arg]),
+            flush=True,
+            file=stdout,
+        )
         if callable(builtins.aliases[arg]):
             builtins.__xonsh__.superhelp(builtins.aliases[arg])
 
@@ -124,7 +128,7 @@ def which(args, stdin=None, stdout=None, stderr=None, spec=None):
     pargs = parser.parse_args(args)
     verbose = pargs.verbose or pargs.all
     if spec is not None:
-        captured = spec.captured in xproc.STDOUT_CAPTURE_KINDS
+        captured = spec.captured in xpp.STDOUT_CAPTURE_KINDS
     else:
         captured = False
     if pargs.plain:
